"""Tests for delete CLI functions"""

import logging

from click import Context
from click.testing import CliRunner

from housekeeper.cli import delete
from housekeeper.store.models import File, Tag
from housekeeper.store.store import Store


def test_delete_non_existing_tag(
    non_existent_tag_name: str,
    base_context: Context,
    cli_runner: CliRunner,
    caplog,
):
    """Test trying to delete a non-existent tag."""
    # GIVEN a tag that does not exist

    # WHEN trying to delete the non-existent tag
    result = cli_runner.invoke(delete.tag_cmd, ["--name", non_existent_tag_name], obj=base_context)

    # THEN assert it exits non zero
    assert result.exit_code == 1
    # THEN it should communicate that the tag was not found
    assert f"Tag {non_existent_tag_name} not found" in caplog.text


def test_delete_existing_tag_with_confirmation(
    populated_context: Context, cli_runner: CliRunner, caplog, family_tag_name: str
):
    """Test deleting an existing tag with confirmation."""
    # GIVEN an existing tag in a populated store
    store: Store = populated_context["store"]
    tag: Tag = store.get_tag(tag_name=family_tag_name)
    assert tag

    # WHEN trying to delete the tag
    result = cli_runner.invoke(delete.tag_cmd, ["--name", family_tag_name], obj=populated_context)

    # THEN the confirmation question should be shown in stdout
    assert f"Delete tag {family_tag_name} with" in result.output


def test_delete_existing_tag_no_confirmation(
    populated_context: Context, cli_runner: CliRunner, caplog, family_tag_name: str
):
    """Test deleting an existing tag without confirmation."""
    # GIVEN an existing tag in a populated store
    caplog.set_level(logging.DEBUG)
    store: Store = populated_context["store"]
    tag: Tag = store.get_tag(tag_name=family_tag_name)
    assert tag

    # WHEN trying to delete the tag
    result = cli_runner.invoke(
        delete.tag_cmd, ["--name", family_tag_name, "--yes"], obj=populated_context
    )

    # THEN the tag should have been deleted
    assert result.exit_code == 0
    assert f"Tag {family_tag_name} deleted" in caplog.text
    assert not store.get_tag(tag_name=family_tag_name)


def test_delete_tag_without_deleting_file(
    base_context: Context,
    populated_store: Store,
    cli_runner: CliRunner,
):
    """Test that deleting a tag does not delete files with the tag."""
    # GIVEN a file with a tag
    file_with_tag: File = populated_store.get_file_by_id(file_id=1)
    assert file_with_tag
    tag: Tag = file_with_tag.tags[0]
    assert tag

    # WHEN deleting the tag
    cli_runner.invoke(delete.tag_cmd, ["--name", tag.name, "--yes"], obj=base_context)

    # THEN the tag is deleted
    assert not populated_store.get_tag(tag_name=tag.name)
    # THEN the file is not deleted
    file_without_tag: File = populated_store.get_files(file_path=file_with_tag.path).first()
    assert file_without_tag
    # THEN the file no longer has the tag
    assert tag not in file_without_tag.tags



"""Tests for the models"""

from housekeeper.store.models import Archive, Bundle, Version
from housekeeper.store.store import Store


def test_instantiate_bundle_obj(case_id, timestamp):
    """Test instantiate a Bundle object"""
    # GIVEN a case id and a time stamp
    # WHEN instantiating a bundle
    bundle_obj = Bundle(name=case_id, created_at=timestamp)
    # THEN assert it was instantiated as expected
    assert bundle_obj.name == case_id
    assert bundle_obj.created_at == timestamp


def test_instantiate_bundle_obj_no_name():
    """Test instantiate a Bundle object without a name"""
    # WHEN instantiating a bundle without a name
    bundle_obj = Bundle()
    # THEN assert it was instantiated as expected
    assert bundle_obj.name is None
    assert bundle_obj.versions == []


def test_version_obj_full_path(project_dir, minimal_bundle_obj):
    """Test returning the full path from a version object"""
    # GIVEN a bundle object
    timestamp = minimal_bundle_obj.created_at
    # WHEN instantiating the version obj
    version_obj = Version(created_at=timestamp, bundle=minimal_bundle_obj)
    # THEN it should point to the correct folder
    root_path = version_obj.full_path
    assert root_path == project_dir / minimal_bundle_obj.name / str(timestamp.date())


def test_delete_file_cascades(archive: Archive, populated_store: Store):
    """Tests that deleting a file deletes the associated archive entry."""

    # GIVEN an archive entry
    archive_task_id: int = archive.archiving_task_id
    file_id: int = archive.file.id

    # WHEN the file is deleted
    populated_store.session.delete(archive.file)
    populated_store.session.commit()

    # THEN the archive entry should be deleted
    for archive_entry in populated_store.get_archives(archival_task_id=archive_task_id):
        assert archive_entry.file.id != file_id


def test_delete_archive_does_not_cascade(archive: Archive, populated_store: Store):
    """Tests that deleting an archive entry does not delete the file entry."""

    # GIVEN an archive entry
    file_path: str = archive.file.path

    # WHEN deleting the archive entry
    populated_store.session.delete(archive)
    populated_store.session.commit()

    # THEN the file should still be in the store
    assert populated_store.get_files(file_path=file_path)



import click
from housekeeper.services.file_report_service.utils import format_files, get_files_table
from housekeeper.store.models import File
from rich.console import Console
import json as jsonlib


class FileReportService:

    def __init__(self, compact: bool = False, json: bool = False):
        self.console = Console()
        self.compact = compact
        self.json = json

    def log_file_table(self, files: list[File], header: str, verbose: bool = False) -> None:
        rows = format_files(files)
        table = get_files_table(
            rows=rows,
            header=header,
            verbose=verbose,
            compact=self.compact,
        )
        self.console.print(table)

        if self.json:
            click.echo(jsonlib.dumps(rows))
